// -*- coding: utf-8  -*-
//
//  AB_NoiseShader.cpp
//  AB_NoiseShader
//
//  Created by André Berg on 2013-10-07.
//  Copyright 2013 Iris VFX. All rights reserved.
//

#include "c4d.h"
#include "c4d_symbols.h"
#include "AB_NoiseShader.h"

#define DEFAULT_COLOR Vector(0.5)
#define DEFAULT_OCTAVES 3.0

class AB_NoiseShaderData : public ShaderData {

    private:
        Vector color;
        Real octaves;

    public:
        virtual Bool Init(GeListNode * node);
        virtual Vector Output(BaseShader * chn, ChannelData * cd);

        virtual INITRENDERRESULT InitRender(BaseShader * sh, const InitRenderStruct& irs);
        virtual void FreeRender(BaseShader * sh);

        static NodeData * Alloc(void) { return gNew AB_NoiseShaderData; }
};

Bool AB_NoiseShaderData::Init(GeListNode * node) {

    BaseContainer * data = ((BaseShader *)node)->GetDataInstance();

    data->SetVector(ABNOISESHADER_COLOR, DEFAULT_COLOR);
    data->SetReal(ABNOISESHADER_OCTAVES, DEFAULT_OCTAVES);

    return true;
}

INITRENDERRESULT AB_NoiseShaderData::InitRender(BaseShader * sh, const InitRenderStruct& irs) {

    BaseContainer * data = sh->GetDataInstance();

    color = data->GetVector(ABNOISESHADER_COLOR);
    octaves = data->GetReal(ABNOISESHADER_OCTAVES);

    return INITRENDERRESULT_OK;
}

void AB_NoiseShaderData::FreeRender(BaseShader * sh) {
    // free owned memory here
}

Vector AB_NoiseShaderData::Output(BaseShader * chn, ChannelData * cd) {

    Vector col = color;
    
    Real oct = octaves;
    Bool abs = true;
    
    Vector pos = cd->p;
    
    Vector turbCol = Vector(Turbulence(pos, oct, abs));

    col ^= turbCol;
    
    return col;
}

// be sure to use a unique ID obtained from www.plugincafe.com
#define ID_ABNOISESHADER   1000001

Bool Register_AB_NoiseShader(void) {
    return RegisterShaderPlugin(ID_ABNOISESHADER, GeLoadString(IDS_ABNOISESHADER), 0, AB_NoiseShaderData::Alloc, "AB_NoiseShader", 0);
}

